/*============================================================================
        SPANNING FOREST of a non necessarily connected, UNDIRECTED graph.
1. Nodes are denoted by integers: 0,...,N-1.
2. The graph is represented as an array G of lists of nodes. In G[i] there is
   a list L of nodes. Node m occurs in L iff there is an arc from i to m.
   An arc (n,m) should have n different from m.
   An undirected arc(n,m) corresponds to two directed arcs: (n,m) and (m,n).
   Thus, it is represented by n occurring in G[m] and m occurring in G[n].
3. The boolean array visited[0..N-1] is such that visited[i]=true iff node i
   has been visited.
4. arcs is a list of pairs of nodes. (n,m) is in arcs iff (n,m) is an arc of
   the spanning forest.
The input graph is given in the file graph2_in whose content is the sequence
of integers:
0 1 0 2 0 3 0 4 0 5 1 2 3 1 5 4 meaning that there is an arc from 0 to 1, an
arc from 1 to 0, an arc from 0 to 2, an arc from 2 to 0, etc.
A different graph is in the file graph3nc_in whose content is the sequence:
0 5 4 0 3 1 5 4
=========================================================================== */
#include <fstream.h>  // file for using files
fstream input_graph;  // file for the input_graph
//----------------------------------------------------------------------------
typedef int node;     // nodes are integers
const int N=6;        // N is the number of nodes in the graph.  N>=2
//----------------------------------------------------------------------------
//                    pair is (node1,node2)
class pair
{node f, s;
 public :
 pair(node a, node b) {f=a; s=b;}
 node fst(){return f;}
 node snd(){return s;}
};
//----------------------------------------------------------------------------
//                    pair_list is (node1,node2) list
class pair_list
{struct pri_cell                                        // pri_cell is private
   {pair atom; pri_cell* next;
    pri_cell(pair at, pri_cell* nx) : atom(at), next(nx) { };};
    pri_cell* ptr;
public :
pair_list(): ptr(NULL){}

pair phead()
{if (ptr == NULL) {cout << "\nError 1: head of empty_list!\n";}
 else {return ptr->atom;} }

void ptail()
{if (ptr == NULL) {cout << "\nError 2: tail of empty_list!\n";}
 else {ptr=ptr->next;} }

void pcons(pair x) {ptr=new pri_cell(x,ptr);}

void printpair_list()
{if (ptr == NULL) {cout << "nil";}
 else {pair ip=ptr->atom; cout <<"("<<ip.fst()<<","<<ip.snd()<<") ";
       pri_cell* ptr1=ptr; ptr=ptr->next;
       if (ptr!=NULL) {printpair_list();}; ptr=ptr1;}}
};    // end of the class pair_list
//----------------------------------------------------------------------------
//                    tuple_list is (node,pred,symm) list
class cell
{public:
  node atom; cell* pred; cell* symm; cell* next;
  cell(node at, cell* pd, cell* sm, cell* nx) :              // cell is public
         atom(at), pred(pd), symm(sm), next(nx) { };
};
typedef cell* tuple_list;        // tuple_list is a pointer to the public cell
//----------------------------------------------------------------------------
// The graph denoted by the global array G of N elements. Each element
// G[0],...,G[N-1} is a tuple_list
tuple_list G[N]={};
//----------------------------------------------------------------------------
node thead(tuple_list L)
{if (L == NULL)
    cout << "\nError 3: head of empty_list!\n";
 else {return L->atom;}}

tuple_list ttail(tuple_list L)
{if (L == NULL)
    {cout << "\nError 4: tail of empty_list!\n";}
 else {if (L->next!=NULL) {(L->next)->pred=NULL;}
      return L->next;}}

tuple_list tcons(node n, node m)
{tuple_list L=new cell(n,NULL,NULL,G[m]);
if (G[m]!=NULL) {G[m]->pred=L;};          // <---- setting the predecessor arc
tuple_list Ls=new cell(m,NULL,NULL,G[n]); // <---- setting the symmetric arc
if (G[n]!=NULL) {G[n]->pred=Ls;};         //
L->symm=Ls; Ls->symm=L;                   // <---- setting the symm pointer
G[n]=Ls; return L;}

void printtuple_list(tuple_list L)
{if (L == NULL) {cout << "nil";}
 else {node p=L->atom; cout << p <<" "; cell* L1=L; L=L->next;
       if (L!=NULL) {printtuple_list(L);}; L=L1;}}
//----------------------------------------------------------------------------
void readgraph()
{int from_node=0, to_node=0;
for (int i=0; i<N-1; i=i+1) {G[i]=NULL;};
input_graph >> from_node;        // <-- eof()=1 if we read a non existing item
while (!input_graph.eof())
     {input_graph >> to_node;
      G[from_node]=tcons(to_node,from_node);
      input_graph >> from_node;     // <-- trying to read the first node
      };}                           //      of a new <from_node, to_node> pair
//----------------------------------------------------------------------------
//                    arcs of the spanning forest: a pair_list
pair_list arcs;
//----------------------------------------------------------------------------
//                    the global array `visited'. Initialized to false
bool visited[N]={};
//----------------------------------------------------------------------------
void visit(node n)
{visited[n]=true;
 while (G[n]!=NULL)
    {node m=thead(G[n]);
     if (visited[m]==false)
          {arcs.pcons(pair(n,m));
           //------------- deletion of the symmetric arc
           cell* p=G[n]->symm;
           if ((p->pred)!=NULL) {(p->pred)->next=p->next;}
           else p=ttail(p);
           //------------- end of deletion of the symmetric arc
           visit(m);};
     G[n]=ttail(G[n]);};                     // the global array G is modified
};
//----------------------------------------------------------------------------
void main()
{cout << "\nThe nodes are: {0,...,"<<N-1<<"}.\n";
input_graph.open("graph2_in", ios::in);
// graph2_in   is undirected and connected
// graph3nc_in is undirected and not connected
readgraph();
cout << "\nThe given undirected graph represented as array of lists, is:";
for (int i=0; i<N; i++)
{cout << "\n G["<<i<<"]= "; printtuple_list(G[i]); }; cout << "\n";
//----------------------------------------------------------------------------
// The for statement is required because the graph may be not connected.
for (int i=0; i<N; i++)
{if (visited[i]==false && G[i]!=NULL) {visit(i);};}
cout << "\nThe undirected arcs of a spanning forest are:\n";
cout << "(Nodes not occurring in the arcs are isolated.)\n";
arcs.printpair_list(); cout << "\n\n";
}
/*----------------------------------------------------------------------------
For the graph2_in the output is:
    The nodes are: {0,...,5}.

    The given undirected graph represented as array of lists, is:
     G[0]= 5 4 3 2 1
     G[1]= 3 2 0
     G[2]= 1 0
     G[3]= 1 0
     G[4]= 5 0
     G[5]= 4 0

    The undirected arcs of a spanning forest are:
    (Nodes not occurring in the arcs are isolated.)
    (1,2) (3,1) (0,3) (5,4) (0,5)
------------------------------------------------------------------------------
For the graph3nc_in the output is:
    The nodes are: {0,...,5}.

    The given undirected graph represented as array of lists, is:
     G[0]= 4 5
     G[1]= 3
     G[2]= nil
     G[3]= 1
     G[4]= 5 0
     G[5]= 4 0

    The undirected arcs of a spanning forest are:
    (Nodes not occurring in the arcs are isolated.)
    (1,3) (4,5) (0,4)
--------------------------------------------------------------------------- */